/** Copyright 2002-2012 CS Systèmes d'Information
 * Licensed to CS Systèmes d'Information (CS) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * CS licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// this file was created by SCC and is largely a derived work from the
// original file AltitudeDetector.java created by CS Systèmes d'Information

package org.orekit.python;

import org.orekit.bodies.BodyShape;
import org.orekit.propagation.SpacecraftState;
import org.orekit.propagation.events.AltitudeDetector;
import org.orekit.errors.OrekitException;

/** Finder for satellite altitude crossing events.
 * <p>This class finds altitude events (i.e. satellite crossing
 * a predefined altitude level above ground).</p>
 * <p>The default implementation behavior is to {@link
 * EventDetector.Action#CONTINUE continue} propagation when ascending and to
 * {@link EventDetector.Action#STOP stop} propagation
 * when descending. This can be changed by overriding the
 * {@link #eventOccurred(SpacecraftState, boolean) eventOccurred} method in a
 * derived class.</p>
 * @see org.orekit.propagation.Propagator#addEventDetector(EventDetector)
 * @author Luc Maisonobe
 */
public class PythonAltitudeDetector extends AltitudeDetector {

    /** Serializable UID. */
    private static final long serialVersionUID = -1552109617025755015L;

    /** Part of JCC Python interface to object */
    private long pythonObject;

    /** Part of JCC Python interface to object */
	public void pythonExtension(long pythonObject)
	{
		this.pythonObject = pythonObject;
	}

    /** Part of JCC Python interface to object */
	public long pythonExtension()
	{
		return this.pythonObject;
	}

    /** Part of JCC Python interface to object */
	public void finalize()
			throws Throwable
			{
		pythonDecRef();
			}

    /** Part of JCC Python interface to object */
	public native void pythonDecRef();

	/* Below this class specifics */

    /** Build a new altitude detector.
     * <p>This simple constructor takes default values for maximal checking
     *  interval ({@link #DEFAULT_MAXCHECK}) and convergence threshold
     * ({@link #DEFAULT_THRESHOLD}).</p>
     * @param altitude threshold altitude value
     * @param bodyShape body shape with respect to which altitude should be evaluated
     */
    public PythonAltitudeDetector(final double altitude, final BodyShape bodyShape) {
        super(altitude, bodyShape);
    }

    /** Build a new altitude detector.
     * <p>This simple constructor takes default value for convergence threshold
     * ({@link #DEFAULT_THRESHOLD}).</p>
     * <p>The maximal interval between altitude checks should
     * be smaller than the half duration of the minimal pass to handle,
     * otherwise some short passes could be missed.</p>
     * @param maxCheck maximal checking interval (s)
     * @param altitude threshold altitude value (m)
     * @param bodyShape body shape with respect to which altitude should be evaluated
     */
    public PythonAltitudeDetector(final double maxCheck,
            final double altitude,
            final BodyShape bodyShape) {
        super(maxCheck, altitude, bodyShape);
    }

    /** Build a new altitude detector.
     * <p>The maximal interval between altitude checks should
     * be smaller than the half duration of the minimal pass to handle,
     * otherwise some short passes could be missed.</p>
     * @param maxCheck maximal checking interval (s)
     * @param threshold convergence threshold (s)
     * @param altitude threshold altitude value (m)
     * @param bodyShape body shape with respect to which altitude should be evaluated
     */
    public PythonAltitudeDetector(final double maxCheck,
            final double threshold,
            final double altitude,
            final BodyShape bodyShape) {
        super(maxCheck, threshold, altitude, bodyShape);
    }

    /** Handle an altitude event and choose what to do next.
     * <p>The default implementation behavior is to {@link
     * EventDetector.Action#CONTINUE continue} propagation when ascending and to
     * {@link EventDetector.Action#STOP stop} propagationwhen descending.</p>
     * @param s the current state information : date, kinematics, attitude
     * @param increasing if true, the value of the switching function increases
     * when times increases around event
     * @return {@link EventDetector.Action#STOP} or {@link EventDetector.Action#CONTINUE}
     * @exception OrekitException if some specific error occurs
     */
    public native Action eventOccurred(final SpacecraftState s, final boolean increasing)
        throws OrekitException;

}
