/** Copyright 2002-2012 CS Systèmes d'Information
 * Licensed to CS Systèmes d'Information (CS) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * CS licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// this file was created by SCC and is largely a derived work from the
// original file DihedralFieldOfViewDetector.java created by CS Systèmes d'Information

package org.orekit.python;

import org.apache.commons.math3.geometry.euclidean.threed.Vector3D;
import org.orekit.propagation.events.CircularFieldOfViewDetector;
import org.orekit.propagation.events.DihedralFieldOfViewDetector;
import org.orekit.propagation.events.EventDetector;
import org.orekit.propagation.events.handlers.EventHandler;
import org.orekit.utils.PVCoordinatesProvider;
import org.orekit.errors.OrekitException;
import org.orekit.propagation.SpacecraftState;

/** Finder for body entering/exiting dihedral FOV events.
 * <p>This class finds dihedral field of view events (i.e. body entry and exit in FOV).</p>
 * <p>The default implementation behavior is to {@link
 * org.orekit.propagation.events.handlers.EventHandler.Action#CONTINUE continue}
 * propagation at entry and to {@link
 * org.orekit.propagation.events.handlers.EventHandler.Action#STOP stop} propagation
 * at exit. This can be changed by calling
 * {@link #withHandler(EventHandler)} after construction.</p>
 * @see org.orekit.propagation.Propagator#addEventDetector(EventDetector)
 * @see CircularFieldOfViewDetector
 * @author V&eacute;ronique Pommier-Maurussane
 */
public class PythonDihedralFieldOfViewDetector extends DihedralFieldOfViewDetector {

    /** Serializable UID. */
    private static final long serialVersionUID = 4571340030201230951L;

    /** Part of JCC Python interface to object */
    private long pythonObject;

    /** Part of JCC Python interface to object */
  	public void pythonExtension(long pythonObject)
  	{
  		this.pythonObject = pythonObject;
  	}

    /** Part of JCC Python interface to object */
  	public long pythonExtension()
  	{
  		return this.pythonObject;
  	}

    /** Part of JCC Python interface to object */
  	public void finalize()
  			throws Throwable
  			{
  		pythonDecRef();
  			}

    /** Part of JCC Python interface to object */
  	public native void pythonDecRef();

    /** Build a new instance.
     * <p>The maximal interval between distance to FOV boundary checks should
     * be smaller than the half duration of the minimal pass to handle,
     * otherwise some short passes could be missed.</p>
     * @param maxCheck maximal interval in seconds
     * @param pvTarget Position/velocity provider of the considered target
     * @param center Direction of the FOV center
     * @param axis1 FOV dihedral axis 1
     * @param halfAperture1 FOV dihedral half aperture angle 1
     * @param axis2 FOV dihedral axis 2
     * @param halfAperture2 FOV dihedral half aperture angle 2
     */
    public PythonDihedralFieldOfViewDetector(final double maxCheck,
            final PVCoordinatesProvider pvTarget, final Vector3D center, final Vector3D axis1, final double halfAperture1,
            final Vector3D axis2, final double halfAperture2) {
        super(maxCheck, pvTarget, center, axis1, halfAperture1, axis2, halfAperture2);
    }

    /** Handle an fov event and choose what to do next.
     * <p>The default implementation behavior is to {@link
     * EventDetector.Action#CONTINUE continue} propagation at entry and to
     * {@link EventDetector.Action#STOP stop} propagation at exit. This can
     * be changed by overriding the {@link #eventOccurred(SpacecraftState, boolean)
     * eventOccurred} method in a derived class.</p>
     * @param s the current state information : date, kinematics, attitude
     * @param increasing if true, the value of the switching function increases
     * when times increases around event, i.e. target enters the fov (note that increase
     * is measured with respect to physical time, not with respect to propagation which
     * may go backward in time)
     * @return one of {@link EventDetector.Action#STOP}, {@link EventDetector.Action#RESET_STATE},
     * {@link EventDetector.Action#RESET_DERIVATIVES} or {@link EventDetector.Action#CONTINUE}
     * @exception OrekitException if some specific error occurs
     */
    public native Action eventOccurred(final SpacecraftState s, final boolean increasing)
        throws OrekitException;

}
