/** Copyright 2014 SSC and 2002-2014 CS Systèmes d'Information
 * Licensed to CS SystÃ¨mes d'Information (CS) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * CS licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// this file was created by SCC and is largely a derived work from the
// original file EclipseDetector.java created by CS Systèmes d'Information

package org.orekit.python;

import org.orekit.utils.PVCoordinatesProvider;
import org.orekit.propagation.events.EclipseDetector;
import org.orekit.propagation.events.EventDetector;
import org.orekit.propagation.events.handlers.EventHandler;
import org.orekit.errors.OrekitException;
import org.orekit.propagation.SpacecraftState;


/** Finder for satellite eclipse related events.
 * <p>This class finds eclipse events, i.e. satellite within umbra (total
 * eclipse) or penumbra (partial eclipse).</p>
 * <p>The default implementation behavior is to {@link
 * org.orekit.propagation.events.handlers.EventHandler.Action#CONTINUE continue}
 * propagation when entering the eclipse and to {@link
 * org.orekit.propagation.events.handlers.EventHandler.Action#STOP stop} propagation
 * when exiting the eclipse. This can be changed by calling {@link
 * #withHandler(EventHandler)} after construction.</p>
 * @see org.orekit.propagation.Propagator#addEventDetector(EventDetector)
 * @author Pascal Parraud
 */
public class PythonEclipseDetector extends EclipseDetector
{

	private static final long serialVersionUID = 1L;

    /** Part of JCC Python interface to object */
	private long pythonObject;

    /** Part of JCC Python interface to object */
	public void pythonExtension(long pythonObject)
	{
		this.pythonObject = pythonObject;
	}

	/** Part of JCC Python interface to object */
    public long pythonExtension()
	{
		return this.pythonObject;
	}

	/** Part of JCC Python interface to object */
    public void finalize()
			throws Throwable
			{
		pythonDecRef();
			}

	/** Part of JCC Python interface to object */
    public native void pythonDecRef();

    /** Build a new eclipse detector.
     * <p>The new instance is a total eclipse (umbra) detector with default
     * values for maximal checking interval ({@link #DEFAULT_MAXCHECK})
     * and convergence threshold ({@link #DEFAULT_THRESHOLD}).</p>
     * @param occulted the body to be occulted
     * @param occultedRadius the radius of the body to be occulted (m)
     * @param occulting the occulting body
     * @param occultingRadius the occulting body radius (m)
     */
	public  PythonEclipseDetector(final PVCoordinatesProvider occulted,
			final double occultedRadius,
			final PVCoordinatesProvider occulting,
			final double occultingRadius)
	{
		super(occulted, occultedRadius, occulting, occultingRadius);
	}

	 /** Build a new eclipse detector.
     * <p>The new instance is a total eclipse (umbra) detector with default
     * value for convergence threshold ({@link #DEFAULT_THRESHOLD}).</p>
     * <p>The maximal interval between eclipse checks should be smaller than
     * the half duration of the minimal pass to handle, otherwise some short
     * passes could be missed.</p>
     * @param maxCheck maximal checking interval (s)
     * @param occulted the body to be occulted
     * @param occultedRadius the radius of the body to be occulted in meters
     * @param occulting the occulting body
     * @param occultingRadius the occulting body radius in meters
     */
	public  PythonEclipseDetector(final double maxCheck,
			final PVCoordinatesProvider occulted,
			final double occultedRadius,
			final PVCoordinatesProvider occulting,
			final double occultingRadius)
	{
		super(maxCheck,occulted,occultedRadius,occulting,occultingRadius);
	}

	 /** Build a new eclipse detector.
     * <p>The new instance is a total eclipse (umbra) detector.</p>
     * <p>The maximal interval between eclipse checks should be smaller than
     * the half duration of the minimal pass to handle, otherwise some short
     * passes could be missed.</p>
     * @param maxCheck maximal checking interval (s)
     * @param threshold convergence threshold (s)
     * @param occulted the body to be occulted
     * @param occultedRadius the radius of the body to be occulted in meters
     * @param occulting the occulting body
     * @param occultingRadius the occulting body radius in meters
     */
	public  PythonEclipseDetector(final double maxCheck,
			final double threshold,
			final PVCoordinatesProvider occulted,
			final double occultedRadius,
			final PVCoordinatesProvider occulting,
			final double occultingRadius)
	{
		super(maxCheck,threshold,occulted, occultedRadius, occulting, occultingRadius);
	}

	public  PythonEclipseDetector(final double maxCheck,
			final double threshold,
			final PVCoordinatesProvider occulted,
			final double occultedRadius,
			final PVCoordinatesProvider occulting,
			final double occultingRadius,
			final boolean totalEclipse)
	{
		super(maxCheck, threshold, occulted, occultedRadius, occulting, occultingRadius, totalEclipse);
	}

	public native Action eventOccurred(final SpacecraftState s, final boolean increasing) throws OrekitException;

}

