/** Copyright 2014 SSC and 2002-2014 CS Systèmes d'Information
 * Licensed to CS SystÃ¨mes d'Information (CS) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * CS licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// this file was created by SCC and is largely a derived work from the
// original file ElevationDetector.java created by CS Systèmes d'Information

package org.orekit.python;

import org.orekit.frames.TopocentricFrame;
import org.orekit.models.AtmosphericRefractionModel;
import org.orekit.propagation.events.ElevationDetector;
import org.orekit.propagation.events.EventDetector;
/*import org.orekit.propagation.events.handlers.EventHandler;*/
import org.orekit.utils.ElevationMask;
import org.orekit.errors.OrekitException;
import org.orekit.propagation.SpacecraftState;

/**
 * Finder for satellite raising/setting events that allows for the
 * setting of azimuth and/or elevation bounds or a ground azimuth/elevation
 * mask input. Each calculation be configured to use atmospheric refraction
 * as well.
 * <p>The default implementation behavior is to {@link
 * org.orekit.propagation.events.handlers.EventHandler.Action#CONTINUE continue}
 * propagation at raising and to {@link
 * org.orekit.propagation.events.handlers.EventHandler.Action#STOP stop} propagation
 * at setting. This can be changed by calling
 * {@link #withHandler(EventHandler)} after construction.</p>
 * @author Hank Grabowski
 * @since 6.1
 */

public class PythonElevationDetector extends ElevationDetector
{

	private static final long serialVersionUID = -8444573139251039785L;

    /** Part of JCC Python interface to object */
	private long pythonObject;

    /** Part of JCC Python interface to object */
	public void pythonExtension(long pythonObject)
	{
		this.pythonObject = pythonObject;
	}

    /** Part of JCC Python interface to object */
	public long pythonExtension()
	{
		return this.pythonObject;
	}

    /** Part of JCC Python interface to object */
	public void finalize()
			throws Throwable
			{
		pythonDecRef();
			}

    /**
     * Creates an instance of Elevation detector based on passed in topocentric frame
     * and the minimum elevation angle.
     * <p>
     * uses default values for maximal checking interval ({@link #DEFAULT_MAXCHECK})
     * and convergence threshold ({@link #DEFAULT_THRESHOLD}).</p>
     * @param topo reference to a topocentric model
     * @see #withConstantElevation(double)
     * @see #withElevationMask(ElevationMask)
     * @see #withRefraction(AtmosphericRefractionModel)
     */
	public PythonElevationDetector(TopocentricFrame topo) {
		super(topo);
	}


    /**
     * Creates an instance of Elevation detector based on passed in topocentric frame
     * and the minimum elevation angle.
     * <p>
     * uses default values for maximal checking interval ({@link #DEFAULT_MAXCHECK})
     * and convergence threshold ({@link #DEFAULT_THRESHOLD}).</p>
     * @param minElevation minimum elevation angle
     * @param topo reference to a topocentric model
     * @deprecated as of 6.1 replace with {@link #ElevationDetector(TopocentricFrame)} followed
     * by a call to {@link #withConstantElevation(double)}
     */
	public PythonElevationDetector(double elevation, TopocentricFrame topo) {
		super(elevation, topo);
	}

	 /**
     * Creates an instance of Elevation detector based on passed in topocentric frame
     * and overrides of default maximal checking interval and convergence threshold values.
     * @param maxCheck maximum checking interval (s)
     * @param threshold maximum divergence threshold (s)
     * @param topo reference to a topocentric model
     * @see #withConstantElevation(double)
     * @see #withElevationMask(ElevationMask)
     * @see #withRefraction(AtmosphericRefractionModel)
     */
	public PythonElevationDetector(final double maxCheck, final double threshold, final TopocentricFrame topo) {
		super(maxCheck, threshold, topo);
	}

    /** Handle an event and choose what to do next.

     * <p>The scheduling between this method and the {@link
     * org.orekit.propagation.sampling.OrekitStepHandler OrekitStepHandler} method {@link
     * org.orekit.propagation.sampling.OrekitStepHandler#handleStep(
     * org.orekit.propagation.sampling.OrekitStepInterpolator, boolean)
     * handleStep(interpolator, isLast)} is to call this method first and
     * <code>handleStep</code> afterwards. This scheduling allows the propagator to
     * pass <code>true</code> as the <code>isLast</code> parameter to the step
     * handler to make it aware the step will be the last one if this method
     * returns {@link EventDetector.Action#STOP}. As the interpolator may be used to navigate back
     * throughout the last step (as {@link
     * org.orekit.propagation.sampling.OrekitStepNormalizer OrekitStepNormalizer}
     * does for example), user code called by this method and user
     * code called by step handlers may experience apparently out of order values
     * of the independent time variable. As an example, if the same user object
     * implements both this {@link EventDetector EventDetector} interface and the
     * {@link org.orekit.propagation.sampling.OrekitFixedStepHandler OrekitFixedStepHandler}
     * interface, a <em>forward</em> integration may call its
     * <code>eventOccurred</code> method with a state at 2000-01-01T00:00:10 first
     * and call its <code>handleStep</code> method with a state at 2000-01-01T00:00:09
     * afterwards. Such out of order calls are limited to the size of the
     * integration step for {@link
     * org.orekit.propagation.sampling.OrekitStepHandler variable step handlers} and
     * to the size of the fixed step for {@link
     * org.orekit.propagation.sampling.OrekitFixedStepHandler fixed step handlers}.</p>

     * @param s the current state information : date, kinematics, attitude
     * @param increasing if true, the value of the switching function increases
     * when times increases around event (note that increase is measured with respect
     * to physical time, not with respect to propagation which may go backward in time)
     * @return one of {@link EventDetector.Action#STOP}, {@link EventDetector.Action#RESET_STATE},
     * {@link EventDetector.Action#RESET_DERIVATIVES} or {@link EventDetector.Action#CONTINUE}
     * @exception OrekitException if some specific error occurs
     * @deprecated as of 6.1 replaced by {@link
     * org.orekit.propagation.events.handlers.EventHandler#eventOccurred(SpacecraftState,
     * EventDetector, boolean)}
     */
	public native Action eventOccurred(final SpacecraftState s, final boolean increasing) throws OrekitException;

	public native void pythonDecRef();

}

